#include "ti86asm.inc"

_user_int_ram .equ $D2FD

.org _asm_exec_ram

   call _flushallmenus		;exit all menus
   call _clrScrn			;clear the screen and text shaddow
   call _homeup				;move the cursor to the top left
   bit 2,(iy+$23)		;check if a user interupt routine is running
   jr nz,OFF			;if it is, turn it off and exit
   ld hl,TextON			;if it's not
   call _puts			;display "ScreenSaver Enabled"

   res 2,(iy+23)	;turn off the user int while we copy the new one
   ld hl,int
   ld de,_user_int_ram+1
   ld bc,int_end-int
   ldir					;copy the file to the interrupt mem
   
   ld a,(_user_int_ram+1)		;set up the checksum byte... 
   ld hl,_user_int_ram+($28*1)
   add a,(hl)
   ld hl,_user_int_ram+($28*2)
   add a,(hl)
   ld hl,_user_int_ram+($28*3)
   add a,(hl)
   ld hl,_user_int_ram+($28*4)
   add a,(hl)
   ld hl,_user_int_ram+($28*5)
   add a,(hl)
   ld (_user_int_ram),a		;save the checksum byte

   xor a			;set up some of the variables used by the SS
   ld (Temp+2),a
   ld (Temp+4),a
   ld a,2
   ld (Temp+3),a

   set 2,(iy+$23)	;turn on the interrupt
   ret				;exit from this program and leave the interrupt
					;program (screen saver) running
OFF:
   ld hl,TextOFF
   call _puts		;display "ScreenSaver Disabled
   res 2,(iy+$23)	;turn it off
   ret				;exit

TextON:
  .db "ScreenSaver Enabled  ",0
TextOFF:
  .db "ScreenSaver Disabled ",0


;sorry for the lack of comments in the interrupt program but it gets
;too confusing for me to explain so I'll just briefly explain it
;here.  Temp is where the variables start.  The first 2 are used as
;a random pointer for random data out of the calcs ram (they are
;incremented everytime the program is executed - approx 185 Hz)
;the next byte is 0 when the SS is not running and 1 when it is.  The
;4th byte is used as the delay so the screen doesn't scroll too fast
;and the 5th byte of Temp is used to show whether F1&F2 were pressed
;to trigger the SS.

;FYI:  User interrupt programs CANNOT be larger than 200 bytes or
;      they will most likely crash (there is only 200 bytes set aside
;      for each one).

int:
   push af  	;You need to push all the registers
   push bc		;you use in the program.
   push de		;if you don't use them all, you don't
   push hl		;need to push and pop them all

 ld hl,Temp		;decrease the random pointer
 dec (hl)
 jr nc,ok
 inc hl
 dec (hl)
ok:
 ld a,($C00A)
 dec a			;check if the apd counter is 0
 jr nc,SSok
 ld a,$3C		;if it is, reset the video mem so that when
 out (0),a		;the calc is turned back on (it'll crash if we don't)
 jr AD2			;all done...
SSok:
 cp $65			;if the apd counter is less than $65,
 jr c,Start		;start (or continue) the SS
 ld hl,Temp+4
 xor a
 cp (hl)		;if F2&F3 triggered the SS, we have to put the APD
 jr nz,APD_DOWN	;timer back down because the OS resets it from the
				;key press
 ld a,%00111111
 out (1),a
 in a,(1)
 cp %11110011	;if F1 & Second are pressed (at the same time),
 jr z,APD_DOWN	;lower the APD timer and start the SS
 ld a,$3C		;if the SS isn't running,
 out (0),a		;make sure the video memory is normal
 ld hl,Temp+2
 xor a
 cp (hl)		;by now the SS is done, but it has to be shut off
 jr z,AD2		;if the SS hasn't run, we're done
 jr SSoff		;if it has, we have to turn it off

APD_DOWN:
 ld a,$65			;start the screen saver
 ld ($C00A),a		;drop the APD timer to where the screen
 ld (hl),a			;saver runs at
Start:
 ld hl,Temp+2
 xor a
 cp (hl)		;if this is the first time the SS has been run...
 jr nz,running
 ld (hl),1
 ld hl,$FC00
 ld de,$CA00
 ld bc,1024
 ldir			;copy the normal screen to the graph screen mem

running:
 ld hl,Temp+3
 dec (hl)			;delay (the int prgm must be run 2x before the
 jr nz,ALL_DONE		;screen is scrolled)
 ld (hl),2
 ld a,$0A
 out (0),a

 ld de,$CA00		;rotate the screen up...
 ld hl,$CA10
 ld bc,1008
 ldir				;end rotate


 jr SkipLabel

AD2:			;we can only use jr's in the interrupt prgm so
 jr ALL_DONE	;this is here for the top of the program that
				;can't reach the label ALL_DONE with a jr

SkipLabel:
 ld b,16		;this section fills the bottom line of the screen
 ld de,$CDF0	;with random data for the cheezy effect
loop1:
 ld hl,Temp
 dec (hl)
 jr nc,ok2
 inc hl
 dec (hl)
 dec hl
ok2:
 ld a,(hl)		;hl points to Temp
 ld c,a
 inc hl
 ld l,(hl)
 ld h,c
 ld c,(hl)
 ld a,r
 rla
 xor c
 ld (de),a
 inc de
 djnz loop1		;loop 16 times for each byte of the last row

 ld a,%00111111
 out (1),a
 in a,(1)
 bit 3,a			;if F2 or...
 jr z,ALL_DONE
 bit 2,a			;if F3 is pressed,
 jr z,ALL_DONE		;don't check for a key press...
 ld a,%10000000
 out (1),a
 in a,(1)
 cp $FF				;if any key was NOT pressed,
 jr z,ALL_DONE		;we're done

SSoff:		;this turns off the screen saver and restores the screen
 xor a
 ld (Temp+4),a
 ld a,$3C
 out (0),a		;video mem back to normal
 ld hl,Temp+2
 xor a
 cp (hl)
 jr z,ALL_DONE
 ld (hl),a
				;tell the calc the graph is "dirty" ...
 set graphdraw,(iy+graphflags) 
 ld a,$A1
 ld ($C00A),a	;reset the apd counter

ALL_DONE:
   pop hl
   pop de
   pop bc
   pop af
   ret

Temp .equ $D3C0		;5 bytes before the end or the int ram
int_end:

.end
